#
# This file is a part of the CaosDB Project.
#
# Copyright (C) 2022 IndiScale GmbH <info@indiscale.com>
# Copyright (C) 2022 Florian Spreckelsen <f.spreckelsen@indiscale.com>
# Copyright (C) 2022 Daniel Hornung <d.hornung@indiscale.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program. If not, see <https://www.gnu.org/licenses/>.
#

# @review Daniel Hornung 2022-02-18

import os
import pytest

import caosdb as db
from caosadvancedtools.models.parser import (parse_model_from_json_schema,
                                             JsonSchemaDefinitionError)

FILEPATH = os.path.join(os.path.dirname(
    os.path.abspath(__file__)), 'json-schema-models')


def test_rt_with_string_properties():
    """Test datamodel parsing of datamodel_string_properties.schema.json"""
    # @author Florian Spreckelsen
    # @date 2022-02-17

    model = parse_model_from_json_schema(
        os.path.join(FILEPATH,
                     "datamodel_string_properties.schema.json"))
    assert "Dataset" in model
    dataset_rt = model["Dataset"]
    assert isinstance(dataset_rt, db.RecordType)
    assert dataset_rt.name == "Dataset"
    assert dataset_rt.description == ""
    assert len(dataset_rt.get_properties()) == 4

    assert dataset_rt.get_property("title") is not None
    assert dataset_rt.get_property("campaign") is not None
    assert dataset_rt.get_property("method") is not None

    assert dataset_rt.get_property("The title") is not None
    assert dataset_rt.get_property("titled") is None

    title_prop = dataset_rt.get_property("title")
    assert title_prop.datatype == db.TEXT
    assert dataset_rt.get_importance(title_prop.name) == db.OBLIGATORY

    campaign_prop = dataset_rt.get_property("campaign")
    assert campaign_prop.datatype == db.TEXT
    assert dataset_rt.get_importance(campaign_prop.name) == db.RECOMMENDED

    method_prop = dataset_rt.get_property("method")
    assert method_prop.datatype == db.TEXT
    assert dataset_rt.get_importance(method_prop.name) == db.RECOMMENDED


def test_datamodel_with_atomic_properties():
    """Test read-in of two separate record types with atomic-typed properties."""
    # @author Florian Spreckelsen
    # @date 2022-02-18

    model = parse_model_from_json_schema(os.path.join(
        FILEPATH, "datamodel_atomic_properties.schema.json"))
    assert "Dataset1" in model
    assert "Dataset2" in model

    rt1 = model["Dataset1"]
    assert isinstance(rt1, db.RecordType)
    assert rt1.name == "Dataset1"
    assert rt1.description == "Some description"
    assert len(rt1.get_properties()) == 3

    assert rt1.get_property("title") is not None
    assert rt1.get_property("campaign") is not None
    assert rt1.get_property("1") is not None

    title_prop = rt1.get_property("title")
    assert title_prop.datatype == db.TEXT
    assert rt1.get_importance(title_prop.name) == db.OBLIGATORY

    campaign_prop = rt1.get_property("campaign")
    assert campaign_prop.datatype == db.TEXT
    assert rt1.get_importance(campaign_prop.name) == db.RECOMMENDED

    float_prop = rt1.get_property("1")
    assert float_prop.datatype == db.DOUBLE
    assert rt1.get_importance(float_prop.name) == db.OBLIGATORY

    rt2 = model["Dataset2"]
    assert isinstance(rt2, db.RecordType)
    assert rt2.name == "Dataset2"
    assert not rt2.description
    assert len(rt2.get_properties()) == 3

    date_prop = rt2.get_property("date")
    assert date_prop.datatype == db.DATETIME

    int_prop = rt2.get_property("integer")
    assert int_prop.datatype == db.INTEGER
    assert int_prop.description == "Some integer property"

    bool_prop = rt2.get_property("boolean")
    assert bool_prop.datatype == db.BOOLEAN


def test_required_no_list():
    """Exception must be raised when "required" is not a list."""
    # @author Daniel Hornung
    # @date 2022-02-18

    with pytest.raises(JsonSchemaDefinitionError) as err:
        parse_model_from_json_schema(
            os.path.join(FILEPATH,
                         "datamodel_required_no_list.schema.json"))
    assert "'Dataset' is not of type 'array'" in str(err.value)
